import _extends from 'babel-runtime/helpers/extends';
import _debounce from 'lodash/debounce';

var _desc, _value, _class;

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import classNames from 'classnames';

import { autobind } from 'core-decorators';
import { MentionsInput, Mention } from 'react-mentions';
import { setUserTyping, submitActivity, storeActivityText } from '@ciscospark/redux-module-activity';
import { searchForUser } from '@ciscospark/redux-module-search';
import PresenceAvatar from '@ciscospark/react-container-presence-avatar';
import FileUploader from '@ciscospark/react-container-file-uploader';

import { blurTextArea, focusTextArea } from './actions';
var styles = {
  'messageComposer': 'messageComposer--TwXLs',
  'hasFocus': 'hasFocus--idbd2',
  'addFileContainer': 'addFileContainer--3nVF7',
  'textarea': 'textarea--2l5EU',
  'textareaContainerBig': 'textareaContainerBig--3SMEc'
}; // @related-file ./styles.css

var mentionStyles = {
  'mentions': 'mentions--2BWzJ',
  'mentions__control': 'mentions__control--2l6GS',
  'mentions__input': 'mentions__input--307_s',
  'mentions--singleLine': 'mentions--singleLine--bJL-y',
  'mentions__higlighter': 'mentions__higlighter--RxtNY',
  'mentions--multiLine': 'mentions--multiLine--3lraX',
  'mentions__highlighter': 'mentions__highlighter--2Fg8-',
  'mentions__suggestions': 'mentions__suggestions--2axdk',
  'mentions__suggestions__list': 'mentions__suggestions__list--6vEz6',
  'mentions__suggestions__item': 'mentions__suggestions__item--KdkNq',
  'content': 'content--CdpPd',
  'avatar': 'avatar--24BHf',
  'highlighted-display': 'highlighted-display--1FFGD',
  'mentions__suggestions__item--focused': 'mentions__suggestions__item--focused--2JrXz',
  'mentions__mention': 'mentions__mention--3YDNh'
}; // @related-file ./mentions.css

const injectedPropTypes = {
  activity: PropTypes.object.isRequired,
  avatar: PropTypes.object.isRequired,
  conversation: PropTypes.object.isRequired,
  sparkInstance: PropTypes.object.isRequired,
  users: PropTypes.object.isRequired,
  currentUser: PropTypes.object.isRequired,
  messageComposer: PropTypes.object.isRequired,
  blurTextArea: PropTypes.func.isRequired,
  focusTextArea: PropTypes.func.isRequired,
  setUserTyping: PropTypes.func.isRequired,
  submitActivity: PropTypes.func.isRequired,
  searchForUser: PropTypes.func.isRequired,
  storeActivityText: PropTypes.func.isRequired
};

const propTypes = _extends({
  onSubmit: PropTypes.func.isRequired,
  placeholder: PropTypes.string,
  showMentions: PropTypes.bool
}, injectedPropTypes);

const defaultProps = {
  placeholder: '',
  showMentions: false
};

export let MessageComposer = (_class = class MessageComposer extends Component {
  constructor(props) {
    super(props);
    this.timerId = 0;
    this.setUserStartTyping = _debounce(this.setUserStartTyping, 500, { leading: true, trailing: false });
  }

  shouldComponentUpdate(nextProps) {
    const { props } = this;
    return props.activity !== nextProps.activity || props.placeholder !== nextProps.placeholder;
  }

  setUserTyping(state) {
    const { props } = this;
    const {
      conversation,
      sparkInstance
    } = props;
    props.setUserTyping(state, conversation, sparkInstance);
  }

  setUserStopTyping() {
    this.setUserTyping(false);
  }

  setUserStartTyping() {
    this.setUserTyping(true);
  }

  handleTextChange(e, newValue, newValuePlainText) {
    const { props } = this;
    let { value } = e.target;
    if (newValue && newValue.length) {
      value = newValue;
    }
    props.storeActivityText(value, newValuePlainText);

    clearTimeout(this.timerId);
    if (value === '') {
      this.setUserTyping(false);
    } else {
      if (!props.activity.getIn(['status', 'isTyping'])) {
        this.setUserStartTyping();
      }
      this.timerId = setTimeout(this.setUserStopTyping, 3000);
    }
  }

  handleKeyDown(e) {
    if (e.keyCode === 13 && !e.shiftKey && !e.altKey && !e.ctrlKey && !e.metaKey) {
      this.handleSubmit();
      e.preventDefault();
    }
  }

  handleSubmit() {
    const { props } = this;
    const {
      activity,
      conversation,
      sparkInstance,
      currentUser,
      onSubmit
    } = props;

    if (activity.get('text').length === 0 && activity.get('files').size === 0) {
      return;
    }
    this.setUserTyping(false);
    props.submitActivity(conversation, activity, currentUser, sparkInstance);

    if (typeof onSubmit === 'function') {
      onSubmit();
    }
  }

  handleTextAreaBlur() {
    const { props } = this;
    const {
      conversation,
      sparkInstance
    } = props;
    props.blurTextArea();
    props.setUserTyping(false, conversation, sparkInstance);
  }

  handleTextAreaFocus() {
    this.props.focusTextArea();
  }

  searchForMention(term) {
    const {
      avatar,
      conversation,
      currentUser
    } = this.props;
    const participants = [];
    if (conversation.has('participants') && conversation.get('participants').count()) {
      conversation.get('participants').toJS().forEach(p => {
        if (currentUser.id !== p.id) {
          participants.push({
            display: p.displayName,
            id: p.id,
            avatarUrl: avatar.getIn(['items', p.id])
          });
        }
      });

      if (term) {
        return participants.filter(p => p.display.toLowerCase().includes(term.toLowerCase()));
      }
    }
    return participants;
  }

  static renderSuggestion(entry, search, highlightedDisplay) {
    return React.createElement(
      'div',
      { className: mentionStyles.content },
      React.createElement(
        'div',
        { className: mentionStyles.avatar },
        React.createElement(PresenceAvatar, { avatarId: entry.id, name: entry.display })
      ),
      React.createElement(
        'div',
        { className: mentionStyles.highlightedDisplay },
        highlightedDisplay
      )
    );
  }

  render() {
    let text;
    const { props } = this;
    const {
      activity,
      messageComposer,
      placeholder,
      conversation
    } = this.props;

    if (activity && activity.has('text')) {
      text = activity.get('text');
    }

    const textAreaFocusStyle = messageComposer.getIn(['status', 'hasTextAreaFocus']) ? styles.hasFocus : '';
    const mentionMarkup = '@{__display__}|__id__|';

    // Only show mentions if this is not a one on one convo
    const showMentions = props.showMentions && !conversation.getIn(['status', 'isOneOnOne']);
    const getData = showMentions ? this.searchForMention : () => {};
    return React.createElement(
      'div',
      { className: classNames('ciscospark-message-composer', styles.messageComposer, textAreaFocusStyle) },
      React.createElement(FileUploader, { onSubmit: this.handleSubmit }),
      React.createElement(
        'div',
        { className: classNames('ciscospark-textarea-container') },
        React.createElement(
          MentionsInput,
          {
            classNames: mentionStyles,
            markup: mentionMarkup,
            onBlur: this.handleTextAreaBlur,
            onChange: this.handleTextChange,
            onFocus: this.handleTextAreaFocus,
            onKeyDown: this.handleKeyDown,
            onSubmit: this.handleSubmit,
            placeholder: placeholder,
            rows: 1,
            value: text
          },
          React.createElement(Mention, {
            className: mentionStyles.mentions__mention,
            data: getData,
            renderSuggestion: MessageComposer.renderSuggestion,
            trigger: '@'
          })
        )
      )
    );
  }
}, (_applyDecoratedDescriptor(_class.prototype, 'setUserTyping', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'setUserTyping'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'setUserStopTyping', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'setUserStopTyping'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextChange', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextChange'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleKeyDown', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleKeyDown'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleSubmit', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleSubmit'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextAreaBlur', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextAreaBlur'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextAreaFocus', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextAreaFocus'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'searchForMention', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'searchForMention'), _class.prototype)), _class);

MessageComposer.propTypes = propTypes;
MessageComposer.defaultProps = defaultProps;

function mapStateToProps(state, ownProps) {
  return {
    activity: state.activity,
    avatar: state.avatar,
    conversation: state.conversation,
    sparkInstance: ownProps.sparkInstance || state.spark.get('spark'),
    users: state.users,
    messageComposer: state.messageComposer,
    currentUser: state.users.getIn(['byId', state.users.get('currentUserId')])
  };
}

export default connect(mapStateToProps, dispatch => bindActionCreators({
  blurTextArea,
  focusTextArea,
  setUserTyping,
  submitActivity,
  searchForUser,
  storeActivityText
}, dispatch))(MessageComposer);